open Ast;;
open Printf;;
open Pprint;;
open Solve;;

if Array.length Sys.argv <> 2 then
  (printf "Usage: plog <file>\n";
   exit 0);;

(* Parse the given file and build the syntax tree (AST)
 * for the command that represents the program. *)
let file = open_in (Sys.argv.(1));;
let lexbuf = Lexing.from_channel file;;
let prog, goals = try Parser.program Lexer.token lexbuf
          with Parsing.Parse_error ->
             let pos = lexbuf.Lexing.lex_curr_p in
             printf "Syntax error at line %d\n"
                    pos.Lexing.pos_lnum;
             exit 1;;

(* Pretty-print the program *)
printf "Program:\n";;
Pprint.pprintProg prog;;
printf "\n";;

printf "Goals:\n";;
Pprint.pprintList goals;;
printf ".\n\n";;

(* Collect all variables in the goals *)
let rec vars ac t = match t with
  | Var v -> if List.mem t ac then ac else t::ac
  | Term(n,l) -> List.fold_left vars ac l

let v = List.fold_left vars [] goals;;

(* find and print all solutions *)
let all_solutions = solve prog goals;;

printf "Found %d solution(s).\n" (List.length all_solutions);;
let printSol(sol) =
  List.iter (fun x -> pprintTerm x; printf " = ";
                      pprintTerm (Solve.substitute sol x); printf "\n") v;;
List.iter (fun sol -> printf "\nSolution: \n"; printSol sol) all_solutions;;