(*
 * Name:   XXXXXXXXXXXXXXXXXXXXXXXXXXXX
 * Net Id: XXXXXX
 *)

open Ast

exception FailUnify
exception NotFound

(* Functional implementation of union-find structures.
 * A union-find structure (aka "unifier") u:term->term maps
 *   Var terms to their parents; u(t) raises NotFound if t
 *   doesn't have a parent (i.e., hasn't been unified).
 * find u t returns the representative of t in u.
 * union u t t' extends u with a new mapping t->t'.
 * empty is the empty unifier. *)
let rec find (u:term -> term) (t: term) : term =
  try find u (u t) with NotFound -> t

let union (u: term -> term) (t: term) (t': term) : term -> term =
  fun x -> if x = t then t' else u x

let empty = fun _ -> raise NotFound


(* fresh(r) replaces all variables with fresh values
 * e.g.,
 fresh(Term("mother", Var "X"),
            [Term("female", Var "X"), Term("parent", Var "X", Var "Y")])

    = Term("mother", Var "_X1"),
           [Term("female", Var "_X1"), Term("parent", Var "_X1", Var "_Y2")])
 *)
let fresh (r:rule) : rule =
  raise (Failure "unimplemented")


(* unify t1 t2 unifies terms t1 and t2
 * Returns a unifier u:term->term such that
 *   u(t) = t' if the Var term t has been unified with term t'
 *   u(t) raises NotFound otherwise.
 * Raises FailUnify if the terms cannot be unified. *)
let unify (t1:term) (t2:term): term->term =
  raise (Failure "unimplemented")


(* Substitute unifications u in the term t. Substitutions are
 * applied recursively in the subterms.
 * Returns: the new term after subsituting the unifications. *)
let rec substitute (u:term -> term) (t: term) : term =
    match t with
    | Var v -> let t' = find u t in
               if  t == t' then t' else substitute u t'
    | Term(n,l) -> Term(n, List.map (substitute u) l)

(* match_rule(r, crt, rest), where r is a rule, crt is the current
 *    goal, and rest are the remaining goals.
 * Matches the current goal g  with the left hand side of rule r.
 * Returns a pair (gs, u) consiting of the new set of goals gs and
 *    the unifier u
 * Raises FailUnify if the rule cannot be matched. *)
let match_rule (r:rule) (crt:term) (rest:term list)
  : term list * (term -> term) =
  let g', gs' = fresh r in
  let u = unify crt g' in
  let gs = List.map (substitute u) (gs'@rest) in
    (gs, u)


(* solve p gs finds all solutions satisfying the goals gs for a program p.
 * Returns: a list of unifiers term->term for all of the solutions of
 * the goals gs. *)
let rec solve (p: rule list) (gs: term list) : (term -> term) list =
  raise (Failure "unimplemented")
